"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assertState_1 = __importDefault(require("./assertState"));
class Batcher {
    fn;
    maxBatchSize;
    q = Array();
    flushing = false;
    constructor(fn, maxBatchSize) {
        this.fn = fn;
        this.maxBatchSize = maxBatchSize;
    }
    async maybeFlush() {
        // We don't need to batch by timers (e.g. debounce), as if there is a lot of activity, it will be efficient because the subsequent batches will be optimally sized, and if there isn't, then it doesn't matter. OTOH, using a timer is far more complex and subtle.
        if (this.flushing) {
            return;
        }
        this.flushing = true;
        while (this.q.length) {
            // https://stackoverflow.com/a/48421425/6249022
            const dq = this.maxBatchSize
                ? this.q.splice(0, this.maxBatchSize)
                : this.q.splice(0);
            const outputs = await this.fn(dq.map((e) => e.input));
            (0, assertState_1.default)(outputs.length === dq.length);
            for (const [i, out] of outputs.entries()) {
                dq[i].resolve(out);
            }
        }
        this.flushing = false;
    }
    async execute(input) {
        return new Promise((resolve) => {
            this.q.push({ resolve, input });
            // Intentionally do not wait for Promise. The function is async just so we can use `await` in it.
            void this.maybeFlush();
        });
    }
}
exports.default = Batcher;
